"""
State:          indyzone - event handlers
State type:     IndyZone::event_handlers_sample::1.0
Description:    IndyZone::event handlers sample::1.0
Author:         IndyZone
Date Created:   December 13, 2022
"""


import hou
import viewerstate.utils as su

class State(object):
    def __init__(self, state_name, scene_viewer):
        self.state_name = state_name
        self.scene_viewer = scene_viewer

        self.node = None
        self.mousecoordinate = [0, 0]

    def onEnter(self,kwargs):
        """ Called on node bound states when it starts
        """
        self.node = kwargs["node"]
        state_parms = kwargs["state_parms"]

        self.log("onEnter")

    def onInterrupt(self, kwargs):
        """ Called when the state is interrupted e.g when the mouse 
        moves outside the viewport
        """
        self.log("onInterrupt")

    def onResume(self, kwargs):
        """ Called when an interrupted state resumes
        """
        self.log("onResume")

    def onExit(self,kwargs):
        """ Called when the state terminates
        """
        state_parms = kwargs["state_parms"]

        self.log("onExit")

    def onMouseEvent(self, kwargs):
        """ Process mouse events
        """
        ui_event = kwargs["ui_event"]
        dev = ui_event.device()

        self.mousecoordinate = [dev.mouseX(), dev.mouseY()]

        modifier = 500

        if dev.isLeftButton():
            modifier = 500
        if dev.isMiddleButton():
            modifier = 1000
        if dev.isRightButton():
            modifier = 2000

        reason = ui_event.reason()

        if reason == hou.uiEventReason.Active:
            self.node.parmTuple("scale").set((self.mousecoordinate[0]/modifier, self.mousecoordinate[1]/modifier, 1))


        #self.log("onMouseEvent")
        #self.log("onMouseEvent", dev.mouseX(), dev.mouseY(), dev.isLeftButton)
        #self.log(dev)

        # Must return True to consume the event
        return False

    def onMouseWheelEvent(self, kwargs):
        """ Process a mouse wheel event
        """
        dev = kwargs["ui_event"].device()
        scroll = dev.mouseWheel()

        smoothparm = self.node.parm("smooth")

        current = smoothparm.evalAsFloat()

        smoothparm.set(float(current + scroll))
        
        print("scroll=", scroll)
        
        self.log("onMouseWheelEvent")
        
        # Must return True to consume the event
        return False


def createViewerStateTemplate():
    """ Mandatory entry point to create and return the viewer state 
        template to register. """

    state_typename = kwargs["type"].definition().sections()["DefaultState"].contents()
    state_label = "indyzone - event handlers"
    state_cat = hou.sopNodeTypeCategory()

    template = hou.ViewerStateTemplate(state_typename, state_label, state_cat)
    template.bindFactory(State)
    template.bindIcon(kwargs["type"].icon())

    return template
